<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\Zf1LoggingGelf
 */

namespace DeubertIt\Zf1LoggingGelf;

use DeubertIt\Zf1LoggingGelf\Log\Writer\Gelf;

/**
 * Class Bootstrap
 * @package DeubertIt\Zf1LoggingGelf
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class Bootstrap
{
    /**
     * @param \Zend_Application_Resource_Log|array $resource
     * @param \Zend_Log|null $log
     * @throws \Zend_Log_Exception
     */
    public static function init($resource, ?\Zend_Log $log = null)
    {
        if (is_null($resource)) {
            return;
        }

        if (is_array($resource)) {
            $config = $resource;
        } else {
            $config = $resource->getOptions();
        }

        if (!$config || !count($config)) {
            return;
        }

        foreach ($config as $options) {
            if (!isset($options['writerName']) || strtolower($options['writerName']) != 'gelf') {
                continue;
            }

            $writerParams = isset($options['writerParams']) ? $options['writerParams'] : [];
            
            /** @var \Zend_Log_Writer_Abstract|null $writer */
            $writer = Gelf::factory($writerParams);

            /** @var \Zend_Log_Filter_Abstract|null $filter */
            $filter = self::getFilter($options);
            if (!is_null($filter)) {
                $writer->addFilter($filter);
            }

            if (!$log) {
                $log = new \Zend_Log();
                $log->addWriter($writer);
                $resource->setLog($log);
            } else {
                $log->addWriter($writer);
            }
        }
    }

    /**
     * @param array $options
     * @return \Zend_Log_Formatter_Abstract|null
     */
    protected static function getFormatter(array $options = [])
    {
        $name = isset($options['formatterName']) ? $options['formatterName'] : null;
        if (!is_null($name)) {
            $className = "Zend_Log_Formatter_{$name}";
            $params = isset($options['formatterParams']) ? $options['formatterParams'] : [];
            $formatter = $className::factory($params);

            return $formatter;
        }

        return null;
    }

    /**
     * @param array $options
     * @return \Zend_Log_Filter_Abstract|null
     */
    protected static function getFilter(array $options = [])
    {
        $name = isset($options['filterName']) ? $options['filterName'] : null;
        if (!is_null($name)) {
            $className = "Zend_Log_Filter_{$name}";
            $params = isset($options['filterParams']) ? $options['filterParams'] : [];
            $filter = $className::factory($params);

            return $filter;
        }

        return null;
    }
}
