<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\Zf1LoggingGelf
 */

namespace DeubertIt\Zf1LoggingGelf;

use Monolog\Logger;

/**
 * Class Log
 * @package DeubertIt\Zf1LoggingGelf
 * 
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class Log extends Logger
{
    /**
     * Adds a log record at an arbitrary level.
     *
     * This method allows for compatibility with common interfaces.
     *
     * @param mixed  $level   The log level
     * @param string $message The log message
     * @param array  $context The log context
     */
    public function log($level, $message, array $context = []): void
    {
        $level = self::toMonologErrorCode($level);

        try
        {
            parent::log($level, $message, $context);
        }
        catch (\Exception $e)
        {
            // unable to connect and so on...
        }
    }

    /**
     * @param int $code
     * @return int|null
     */
    public static function toMonologErrorCode($code)
    {
        $log = [
            \Zend_Log::EMERG => Logger::EMERGENCY,
            \Zend_Log::ALERT => Logger::ALERT,
            \Zend_Log::CRIT => Logger::CRITICAL,
            \Zend_Log::ERR => Logger::ERROR,
            \Zend_Log::WARN => Logger::WARNING,
            \Zend_Log::NOTICE => Logger::NOTICE,
            \Zend_Log::INFO => Logger::INFO,
            \Zend_Log::DEBUG => Logger::DEBUG,
        ];

        return isset($log[$code]) ? $log[$code] : null;
    }
}
