<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\Zf1LoggingGelf
 */

namespace DeubertIt\Zf1LoggingGelf\Log\Formatter;

use Monolog\Formatter\GelfMessageFormatter;
use Monolog\Logger;

/**
 * Class GelfFormatter
 * @package DeubertIt\Zf1LoggingGelf\Log\Formatter
 * 
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class GelfFormatter extends \Zend_Log_Formatter_Abstract
{
    /**
     * @var string
     */
    protected $_format;

    const DEFAULT_FORMAT = '%timestamp% %priorityName% (%priority%): %message%' . PHP_EOL;

    /**
     * Class constructor
     *
     * @param  null|string $format  Format specifier for log messages
     * @return void
     * @throws \Zend_Log_Exception
     */
    public function __construct($format = null)
    {
        if ($format === null) {
            $format = self::DEFAULT_FORMAT . PHP_EOL;
        }

        if (!is_string($format)) {
            require_once 'Zend/Log/Exception.php';
            throw new \Zend_Log_Exception('Format must be a string');
        }

        $this->_format = $format;
    }

    /**
     * @param array|\Zend_Config $config
     * @return \Zend_Log_FactoryInterface
     */
    public static function factory($config)
    {
        $format = null;
        if (null !== $config) {
            if ($config instanceof \Zend_Config) {
                $config = $config->toArray();
            }

            if (array_key_exists('format', $config)) {
                $format = $config['format'];
            }
        }
        return new self($format);
    }

    /**
     * @param array $event
     * @return string
     */
    public function format($event)
    {
        $event['extra'] = [];
        $event['context'] = [];
        $event['datetime'] = isset($event['timestamp']) ? $event['timestamp'] : null;
        $event['level'] = isset($event['priorityName']) ? Logger::toMonologLevel($event['priorityName']) : null;

        $messageFormatter = new GelfMessageFormatter();
        $message = $messageFormatter->format($event);

        return $message;
    }
}
