<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\Zf1LoggingGelf
 */

namespace DeubertIt\Zf1LoggingGelf\Log\Writer;

use DeubertIt\Zf1LoggingGelf\Log;
use Gelf\Publisher;
use Gelf\PublisherInterface;
use Gelf\Transport\AbstractTransport;
use Gelf\Transport\UdpTransport;
use Monolog\Formatter\GelfMessageFormatter;
use Monolog\Handler\GelfHandler;
use GuzzleHttp\Psr7\Uri;

/**
 * Trait GelfTrait
 * @package DeubertIt\Zf1LoggingGelf\Log\Writer
 * 
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
trait GelfTrait
{
    /**
     * @var string|null
     */
    protected $target;

    /**
     * @var string|null
     */
    protected $channel;

    /**
     * @var string|null
     */
    protected $host;

    /**
     * @var Publisher|PublisherInterface|null
     */
    protected $publisher;

    /**
     * @var UdpTransport|AbstractTransport|null
     */
    protected $transport;

    /**
     * @var GelfHandler|null
     */
    protected $handler;

    /**
     * @var Logger|null
     */
    protected $logger;

    /**
     * @return string|null
     */
    public function getTarget()
    {
        return $this->target;
    }

    /**
     * @param string|null $target
     * @return $this
     */
    public function setTarget(string $target = null)
    {
        $this->target = $target;
        return $this;
    }

    /**
     * @return string|null
     */
    public function getChannel()
    {
        return $this->channel;
    }

    /**
     * @param string|null $channel
     * @return $this
     */
    public function setChannel(string $channel = null)
    {
        $this->channel = $channel;
        return $this;
    }

    /**
     * @return string|null
     */
    public function getHost()
    {
        return $this->host;
    }

    /**
     * @param string|null $channel
     * @return $this
     */
    public function setHost(string $host = null)
    {
        $this->host = $host;
        return $this;
    }

    /**
     * @return Publisher|PublisherInterface|null
     */
    public function getPublisher()
    {
        if (is_null($this->publisher)) {
            $this->publisher = new Publisher($this->getTransport());
        }
        return $this->publisher;
    }

    /**
     * @param Publisher|PublisherInterface|null $publisher
     * @return $this
     */
    public function setPublisher(PublisherInterface $publisher = null)
    {
        $this->publisher = $publisher;
        return $this;
    }

    /**
     * @return AbstractTransport|UdpTransport|null
     */
    public function getTransport()
    {
        if (is_null($this->transport)) {
            $target = $this->parseTarget();
            $this->transport = new UdpTransport($target['host'], $target['port']);
        }
        return $this->transport;
    }

    /**
     * @param AbstractTransport|UdpTransport|null $transport
     * @return $this
     */
    public function setTransport($transport)
    {
        $this->transport = $transport;
        return $this;
    }

    /**
     * @return GelfHandler|null
     */
    public function getHandler()
    {
        if (is_null($this->handler)) {
            $this->handler = new GelfHandler($this->getPublisher());
            $this->handler->setFormatter(new GelfMessageFormatter($this->host));
        }
        return $this->handler;
    }

    /**
     * @param GelfHandler|null $handler
     * @return $this
     */
    public function setHandler($handler)
    {
        $this->handler = $handler;
        return $this;
    }

    /**
     * @return Logger|null
     */
    public function getLogger()
    {
        if (is_null($this->logger)) {
            $this->logger = new Log($this->getChannel(), [$this->getHandler()]);
        }
        return $this->logger;
    }

    /**
     * @param Logger|null $logger
     * @return $this
     */
    public function setLogger($logger)
    {
        $this->logger = $logger;
        return $this;
    }

    /**
     * @return array
     */
    protected function parseTarget()
    {
        $data = [
            'protocol' => null,
            'host' => null,
            'port' => null,
        ];

        if (is_null($this->getTarget())) {
            return $data;
        }

        $uri = new Uri($this->getTarget());
        $data['scheme'] = $uri->getScheme();
        $data['host'] = $uri->getHost();
        $data['port'] = $uri->getPort();

        return $data;
    }
}
